from abaqus import *
from abaqusConstants import *
from mesh import *
from CONF_config import *
import datetime
import shutil
import numpy as np
def extract_eigfrecs(odb,job_name,stepname):
    if stepname not in odb.steps.keys():
        raise (KeyError,"Could not find specified step: "+stepname+" in output files for job: "+job_name)
    output = odb.steps[stepname].historyRegions['Assembly ASSEMBLY'].historyOutputs
    return {idx+1:(freq[1],damp[1])for idx, (freq,damp) in enumerate(zip(output['EIGFREQ'].data,output['DAMPRATIO'].data))}

def save_job_result_to_file(path,modelname,mode_bib,mode_shapes,model):
    time=datetime.datetime.now().strftime("%Y-%m-%d %H%M")
    filename=path+"\\results\\"+modelname+".txt"
    archive_file_name=path+"\\results\\archive\\"+time+" "+modelname+".txt"
    f = open(filename,'w')
    f.write(time+'\n')
    for mode,mode_data in mode_bib.items():
        f.write('\n'+"Mode "+str(mode)+'\n')
        f.write("\tFreq\t\t"+str(format(mode_data[0],".4f"))+"\tHz"+'\n')
        f.write("\tDamping\t\t"+str(format(100*mode_data[1],".4f"))+"\t%"+'\n')
        try:
            f.write("\tMAC\t\t"+str(format(mode_shapes[mode]["MAC"],".4f"))+"\t~"+'\n')
            f.write("\n\t\t\t"+"\t".join(["A1X","A2X","A3X","A4X","A1Y","A2Y","A3Y","A4Y","A1Z","A2Z","A3Z","A4Z",])+"\n")
            output_buffer=[str(format(datapoint,".2f")) for datapoint in mode_shapes[mode]["Abaqus"]]
            f.write("\tCalculated\t"+"\t".join(output_buffer)+'\n')
            output_buffer=[str(format(datapoint,".2f")) for datapoint in mode_shapes[mode]["Measured"]]
            f.write("\tMeasured\t"+"\t".join(output_buffer)+'\n')
        except:
            pass
    f.write('\n\nModel information\n')   
    f.write('Materials\n')   
    for key,val in model.materials.items():
        f.write("\t"+key+"\n")
        f.write("\t"+"".join(str(val.elastic.table[0]))+'\n\n')
    f.write('CLT sections\n')
    for key,val in model.sections.items():
        try:
            thickness=sum([layer.thickness for layer in val.layup])
            if thickness:
                f.write("\tType\t"+key+'\n\tt\t'+str(thickness)+'\n\n')
        except:
            pass
    f.write('Connectors\n')
    for key,val in model.sections.items():
        try:
            if val.assembledType:
                f.write("\n\t"+key+'\n')
                for item in val.behaviorOptions:
                    f.write('\t\t'+str(item.behavior)+'\n')
                    f.write('\t\t'+str(item.components)+'\n')
                    f.write('\t\t'+str(item.table[0])+'\n')
        except:
            pass
    f.close()
    print(filename,archive_file_name)
    shutil.copyfile(filename,archive_file_name)

def create_MAC_node_set(wallPart, sensor_data):
    for sensorname, data in sensor_data.items():
        #TCBModel.rootAssembly.instances['Walls'].vertices.getByBoundingSphere(center=data["position"],radius=1)
        outputNode = [wallPart.nodes.getClosest(data["position"])]
        outputNodeArray = mesh.MeshNodeArray(nodes=outputNode)
        wallPart.Set(name=sensorname, nodes=outputNodeArray)


def MAC(odb,mode_dict,jobname):
    def MACalculator(phi_a,phi_e):
        phi_a=np.array(phi_a)
        phi_e=np.array(phi_e)
        MAC_up=abs(np.dot(phi_a,phi_e.T))**2
        MAC_down=(np.dot(phi_a,phi_a.T))*(np.dot(phi_e,phi_e.T))
        MAC_index=MAC_up/MAC_down
        return MAC_index
    frequencyStep = odb.steps['Complex frequency']
    abaqusmodes={}
    for modenumber in range(1,len(frequencyStep.frames)):
        frame=frequencyStep.frames[modenumber]
        _modevector=[]
        for axis_map in [0,1,2]:
            for sensorname in ["A1","A2","A3","A4"]:
                node = odb.rootAssembly.nodeSets[sensorname]
                data=frame.fieldOutputs['U'].getSubset(region=node).values[0].data
                _modevector.append(data[axis_map])
        dom_mode_idx=np.argmax(np.abs(_modevector))
        abaqusmodes[modenumber]=[mode_value/_modevector[dom_mode_idx] for mode_value in _modevector]
    modes={
        1:[],
        2:[],
        3:[],
    }
    for mode in range(1,3+1):
        modes[mode]={
                "Abaqus":abaqusmodes[mode],
                "Measured":mode_dict[mode],
                "MAC":MACalculator(abaqusmodes[mode],mode_dict[mode])
            }
    return modes
