# -*- coding: mbcs -*-

#abaqus cae noGUI=main.py
#   200s

from turtle import width
from unittest import expectedFailure
from abaqus import *
from abaqusConstants import *
from part import *
from material import *
from section import *
from assembly import *
from step import *
from interaction import * 
from load import *
from mesh import *
from optimization import *
from job import * 
from sketch import * 
from visualization import *
from connectorBehavior import *
from regionToolset import *
from odbAccess import *
import sys
import getpass
import time


start_time=time.time()

if getpass.getuser() == "egilm":
    script_folder = 'C:\\Users\\egilm\\NTNU\\o365_Masteroppgave2022 - General\ABAQUS_python_model'
elif getpass.getuser() == "Egil":
    script_folder = 'C:\\Users\\Egil\\NTNU\\o365_Masteroppgave2022 - General\\ABAQUS_python_model'
else:
    script_folder = 'C:\\Users\\emili\\NTNU\\o365_Masteroppgave2022 - ABAQUS_python_model'
sys.path.append(script_folder)


from CONF_config                import *
from UTIL_model                 import *
from UTIL_material              import *
from UTIL_import                import *       
from UTIL_section               import *
from UTIL_assembly              import *
from UTIL_sets                  import *
from UTIL_mesh                  import *
from UTIL_boundaryconditions    import *
from UTIL_connectors            import * 
from UTIL_orientation           import *



def assign_properties(input_file,should_refresh_section):
    start_time=time.time()
    section_header("\tImporting database",start_time)
    model = get_model()
    imported_model_data=import_model_data(model,input_file)
    section_header("\tRefreshing part and instance associations",start_time) 
    imported_model_data['All building elements']= reassociate_assembly_items(model,imported_model_data['All building elements'],MODIFIED_SUFFIX)  

    section_header("\tBoundary Condititions",start_time)
    #Loops through and sees which parts are to be connected to the foundation
    foundation_nodes=get_foundation(
        current_model=model, 
        instance_bib=imported_model_data["All building elements"]
    )
    #Groups all the foundation nodes into a foundation set
    model.rootAssembly.Set(
        name='Foundation', 
        vertices=foundation_nodes)
    #Imposes the boundary condition (Lock displacement)
    model.DisplacementBC(
        amplitude=UNSET,
        createStepName='Initial', 
        distributionType=UNIFORM, 
        fieldName='', 
        localCsys=None, 
        name='Foundation', 
        region=model.rootAssembly.sets['Foundation'], 
        u1=SET, 
        u2=SET, 
        u3=SET, 
        ur1=UNSET, 
        ur2=UNSET, 
        ur3=UNSET
        )
    if should_refresh_section:
        section_header("\tSection",start_time)
        #Loops through the parts in the model tree, and looks up which sections they should have by searching through the wall- and floor-dicts 
        for objectname, instanceobject in imported_model_data['All shell elements'].items():        
            instanceobject.part.Set(
                faces=instanceobject.part.faces, 
                name='Face'
                )
            instanceobject.part.SectionAssignment(
                offset=0.0, 
                offsetField='', 
                offsetType=MIDDLE_SURFACE, 
                region=instanceobject.part.sets['Face'], 
                sectionName=instanceobject.section, 
                thicknessAssignment=FROM_SECTION
                )
            instanceobject.part.engineeringFeatures.NonstructuralMass(
                distribution=VOLUME_PROPORTIONAL, 
                magnitude=instanceobject.distributed_mass, 
                name='Distributed mass', 
                region=instanceobject.part.sets['Face'], 
                units=MASS_PER_AREA
                )
        for objectname, instanceobject in imported_model_data["ColumnsBeams"].items():        
            instanceobject.part.Set(
                edges=instanceobject.part.edges,
                name='CentreLine')
            instanceobject.part.SectionAssignment(
                offset=0.0, 
                offsetField='', 
                offsetType=MIDDLE_SURFACE, 
                region=Region(edges=instanceobject.part.edges), 
                sectionName=instanceobject.section, 
                thicknessAssignment=FROM_SECTION)
            instanceobject.part.assignBeamSectionOrientation(
                method=N1_COSINES, 
                n1=(0.0, 0.0, -1.0), 
                region=instanceobject.part.sets['CentreLine'])
    section_header("\tConnections",start_time)
    for connectorgroup in imported_model_data["Connectors"].values():
        connectorgroup.update_connector_properties(imported_model_data)
    section_header("\tAssigning properties finished",start_time)
if __name__ == '__main__':
    input_file= script_folder+'\\Input\\'+MODELNAME+'.xlsx'
    assign_properties(input_file)