# -*- coding: mbcs -*-
from abaqus import *
from abaqusConstants import *
from part import *
from material import *
from section import *
from assembly import *
from step import *
from interaction import * 
from load import *
from mesh import *
from optimization import *
from job import * 
from sketch import * 
from visualization import *
from connectorBehavior import *
from regionToolset import *
from odbAccess import *
import sys
import getpass
import time


if getpass.getuser() == "egilm":
    script_folder = 'C:\\Users\\egilm\\NTNU\\o365_Masteroppgave2022 - General\ABAQUS_python_model'
elif getpass.getuser() == "Egil":
    script_folder = 'C:\\Users\\Egil\\NTNU\\o365_Masteroppgave2022 - General\\ABAQUS_python_model'
else:
    script_folder = 'C:\\Users\\emili\\NTNU\\o365_Masteroppgave2022 - ABAQUS_python_model'
sys.path.append(script_folder)

import CONF_profiler
from UTIL_orientation           import * 
from CONF_config                import *
from UTIL_import                import *
from UTIL_model                 import *
from UTIL_part                  import *
from UTIL_assembly              import *
from UTIL_sets                  import *
from UTIL_mesh                  import *
from UTIL_partitioning          import *
from UTIL_boundaryconditions    import *
import UTIL_modelbuild
from UTIL_connectors            import * 

import CONF_profiler   #@CONF_profiler.profile

def generate_geometry(input_file,save):
    start_time=time.time()
    section_header("\tFlushing database",start_time)
    flush_database()
    rename_model(MODELNAME)
    model = get_model()
    imported_model_data=import_model_data(model,input_file)

    global_axis_system=create_datum_axis_system(
        current_model=model,
        name="Global coordinate system",
        origin=(0,0,0),
        local_x_dir=(1,0,0),
        local_z_dir=(0,1,0),
        )

    section_header("\tBuilding walls",start_time)  
    UTIL_modelbuild.WallTools.generate_walls(imported_model_data["Walls"])
    
    section_header("\tBuilding floors",start_time)  
    UTIL_modelbuild.FloorTools.generate_floors(imported_model_data["Floors"])
    
    section_header("\tBuilding columns and beams",start_time) 
    UTIL_modelbuild.ColumnBeamTools.generate_columns_and_beams(imported_model_data["ColumnsBeams"])
    
    section_header("\tDefining openings",start_time) 
    UTIL_modelbuild.Openingtools.generate_opening_parts(imported_model_data["Openings"])
    
    section_header("\tGenerating openings",start_time) 
    UTIL_modelbuild.Openingtools.generate_openings(imported_model_data["OpeningPlacement"])

    section_header("\tRefreshing part and instance associations",start_time) 
    imported_model_data["All building elements"]= reassociate_assembly_items(model,imported_model_data["All building elements"],MODIFIED_SUFFIX) 
    imported_model_data["OpeningPlacement"]= reassociate_assembly_items(model,imported_model_data["OpeningPlacement"],MODIFIED_SUFFIX) 
    section_header("\tDetecting elements to cut openings into",start_time) 
    cut_dictionary = UTIL_modelbuild.Openingtools.get_cut_dictionary(imported_model_data["OpeningPlacement"],imported_model_data["All shell elements"])
    section_header("\tCutting openings",start_time)
    imported_model_data["All shell elements"]=UTIL_modelbuild.Openingtools.cut_openings(model,imported_model_data["All shell elements"], cut_dictionary,MODIFIED_SUFFIX)
    make_assembly_independent(model)
    section_header("\tCreating datumplanes",start_time)
    create_datumplane_array(
        current_model=model,
        normal=(1,0,0),
        step=imported_model_data["Connector density"]["Horizontal"],
        length=imported_model_data["Layout"]["X Direction"][-1][-1],
        )
    create_datumplane_array(
        current_model=model,
        normal=(0,1,0),
        step=imported_model_data["Connector density"]["Vertical"],
        length=imported_model_data["Layout"]["Y Direction"][-1][-1],
        )
    create_datumplane_array(
        current_model=model,
        normal=(0,0,1),
        step=imported_model_data["Connector density"]["Horizontal"],
        length=imported_model_data["Layout"]["Z Direction"][-1][-1],
        )
    datumplanes_x,datumplanes_y,datumplanes_z=get_datumplanes(current_model=model)
    section_header("\tPartitioning edges",start_time)
    partition_edges_by_plane_array(
            current_model=model,
            plane_array=datumplanes_x,
            instance_bib=imported_model_data["All shell elements"],
            )
    partition_edges_by_plane_array(
        current_model=model,
        plane_array=datumplanes_y,
        instance_bib=imported_model_data["All shell elements"],
        )
    partition_edges_by_plane_array(
        current_model=model,
        plane_array=datumplanes_z,
        instance_bib=imported_model_data["All shell elements"],
        )
    section_header("\tDetecting all coincident points and intersections",start_time)
    all_nodes,all_t_intersects,all_point_intersects=get_all_nodes(
        current_model=model, 
        shell_bib=imported_model_data["All shell elements"],
        all_bib =imported_model_data["All building elements"],
        )
    section_header("\tPartition faces in T-intersections",start_time)
    partition_t_intersects(
        current_model=model,
        t_intersect_bib=all_t_intersects
    )
    partition_point_intersects(
        current_model=model,
        all_bib=imported_model_data["All building elements"],
        cb_bib=imported_model_data["ColumnsBeams"],
        all_point_intersects=all_point_intersects
        )
    section_header("\tGenerating connector sections",start_time)
    generate_connectors(imported_model_data["Connectors"])
    section_header("\tFinding all pairs of points to build connectors in",start_time)
    wire_bib = get_all_coincident_pairs(
        current_model=model,
        shell_bib=imported_model_data["All shell elements"],
        all_bib =imported_model_data["All building elements"],
        )
    #We flip this dict such that we can group all nodes shared between any given combination of instances
    flipped_wire_bib={}
    for coord, combinations in wire_bib.items():
        for combination in combinations:
            if combination in flipped_wire_bib.keys():
                flipped_wire_bib[combination].append(coord)
            else:
                flipped_wire_bib[combination]=[coord]
    points_to_join={}
    #We loop through the instance-pairs in the dict. The points are all the vertices shared between the instance-pairs.
    for combination, coords in flipped_wire_bib.items():
        points=[(model.rootAssembly.instances[combination[0]].vertices.findAt(coord, ), model.rootAssembly.instances[combination[1]].vertices.findAt(coord, )) for coord in coords]
        a   =  combination[0][0]
        b   =  combination[1][0]
        if (a=="V" and b=="F"):
            #Wall-floor
            [push_value(points_to_join,imported_model_data["Walls"][combination[0]].section+" | hor",points_pair) for points_pair in points]
        if (a=="F" and b=="V"):
            #Wall-floor
            [push_value(points_to_join,imported_model_data["Walls"][combination[1]].section+" | hor",points_pair) for points_pair in points]            
        elif a==b and a== "V":
            #Wall-wall
            #Vertical joint
            if imported_model_data["Walls"][combination[0]].get_bounding_box_span()[1]==imported_model_data["Walls"][combination[1]].get_bounding_box_span()[1]:
                [push_value(points_to_join,imported_model_data["Walls"][combination[0]].section+" | ver",points_pair) for points_pair in points]
        elif a==b and a== "F":
            #Floor-Floor
            [push_value(points_to_join,"Floor-Floor",points_pair) for points_pair in points]
        elif "C" in (a,b):
            #Column/beam connection
            [push_value(points_to_join,"CB",points_pair) for points_pair in points]

    section_header("\tBuilding wire connections between all point pairs",start_time)
    count=0
    edges_bib={}
    for connection_category,points_pairs in points_to_join.items():
        if len(points_pairs)>0:
            count+=1
            edges_bib["Wire-"+str(count)]=(connection_category,[])
            wire=model.rootAssembly.WirePolyLine(
                mergeType   = IMPRINT, 
                meshable    = False, 
                points=points_pairs
                )
            
    #We need to locate and group the edges that were created with the wire feature. So we search by name
    for k in range(0, len(model.rootAssembly.edges)):
        if "Wire-" in model.rootAssembly.edges[k].featureName:
            #Storing the found edge into the edge-dict
            edges_bib[model.rootAssembly.edges[k].featureName][1].append(model.rootAssembly.edges[k:k+1])
    section_header("\tAssigning connectors",start_time)
    for wireset, (connectortype,edges) in edges_bib.items():
        model.rootAssembly.Set(
            edges=edges,
            name="SET "+connectortype
            )
        model.rootAssembly.SectionAssignment(
            region          = model.rootAssembly.sets["SET "+connectortype], 
            sectionName     = connectortype
            )
        model.rootAssembly.ConnectorOrientation(
            localCsys1=model.rootAssembly.datums[1], 
            region=model.rootAssembly.allSets["SET "+connectortype]
            )
        

    section_header("\tOrientation",start_time)
    for assemblyobject in imported_model_data["Walls"].values():
        orientate_shell(
            part=assemblyobject.part,
            axis=AXIS_1,
            datum=global_axis_system
            )
    for assemblyobject in imported_model_data["Floors"].values():
        orientate_shell(
            part=assemblyobject.part,
            axis=AXIS_2,
            datum=global_axis_system
            )
    for assemblyobject in imported_model_data["ColumnsBeams"].values():
        orientate_cb(
            part=assemblyobject.part,
            datum=global_axis_system
            )
    section_header("\tGeometry creation finished",start_time)
    if save:
        get_database().saveAs(pathName=script_folder+"\\pre_generated_models\\"+MODELNAME)
    return model

if __name__=='__main__':
    start_time=time.time()
    input_file= script_folder+'\\Input\\'+MODELNAME+'.xlsx'
    generate_geometry(input_file,save=True)
