# -*- coding: mbcs -*-
from abaqus import *
from abaqusConstants import *
from part import *
from material import *
from section import *
from assembly import *
from step import *
from interaction import *
from load import *
from mesh import *
from optimization import *
from job import *
from sketch import *
from visualization import *
from connectorBehavior import *
from regionToolset import *
from odbAccess import *
from CONF_config import *
def reassociate_assembly_items(model,instance_bib,modified_suffix):
    for instance_object in instance_bib.values():
        if instance_object.partname+modified_suffix in model.parts.keys():
            instance_object.partname        = instance_object.partname+modified_suffix
            instance_object.part            = model.parts[instance_object.partname]
        elif instance_object.partname in model.parts.keys():
            instance_object.part            = model.parts[instance_object.partname]
        else:
            instance_object.part=None

        if instance_object.instancename+modified_suffix in model.rootAssembly.instances.keys():
            instance_object.instancename    = instance_object.instancename+modified_suffix
            instance_object.instance        = model.rootAssembly.instances[instance_object.instancename]
        elif instance_object.instancename in model.rootAssembly.instances.keys():
            instance_object.instance        = model.rootAssembly.instances[instance_object.instancename]
        else:
            instance_object.instance=None
    return instance_bib           
def make_assembly_independent(current_model):
    for instance in current_model.rootAssembly.instances.values():
        try:
            #Make all instances independent at the end (cutting a new part creates a dependent part by default)
            current_model.rootAssembly.makeIndependent((instance,))
        except:
            pass
def create_datum_axis_system(
        current_model=[],
        name="Coordinate system",
        origin=(0,0,0),
        local_x_dir=(1,0,0),
        local_z_dir=(0,0,1),
):
    return (current_model.rootAssembly.DatumCsysByThreePoints(
    coordSysType=CARTESIAN, 
    name=name, 
    origin=origin, 
    point1=(origin[0]+local_x_dir[0], origin[1]+local_x_dir[1], origin[2]+local_x_dir[2]), 
    point2=(origin[0]+local_z_dir[0], origin[1]+local_z_dir[1], origin[2]+local_z_dir[2]))
    )

def create_instance(
    current_model   = [],
    instancename    = [],
    part            = [],

):
    instance=current_model.rootAssembly.Instance(
        dependent=OFF, 
        name=instancename, 
        part=part
        )
    return instance


def move_instance(
    current_model=[],
    instancename    =[],
    translation  =(0,0,0),
    rotation_angle=0,
    rotation_axis = (0,1,0),
):  
    if rotation_angle:
        current_model.rootAssembly.rotate(
            angle=rotation_angle, 
            axisDirection=rotation_axis, 
            axisPoint=(0.0, 0.0, 0.0),
            instanceList=(instancename, ))

    current_model.rootAssembly.translate(
        instanceList=(instancename, ), 
        vector=translation
        )


def get_all_nodes(
    current_model=[],
    shell_bib={},
    all_bib ={},
    ):
    # This function is complete garbage, deal with it. Eats a ton of memory
    all_nodes={}
    for key in all_bib.keys():
        for node in current_model.rootAssembly.instances[key].vertices:
            if node.pointOn[0] in all_nodes.keys():
                all_nodes[node.pointOn[0]].append(key)
            else:
                all_nodes[node.pointOn[0]]=[key]

    all_t_intersects={}
    all_point_intersects={}
    #Loop through all nodes, with associated walls at given nodes
    for node,walls_at_node in all_nodes.items():
        #Loops through the wall instance dict to see wether the node touches any other wall faces
        for shell_name, shell_instance in shell_bib.items():
            #Checks if node is contact with a different walls' bounding box without that given wall being represented at the node
            if shell_instance.detect_overlap(node)>1 and not shell_name in walls_at_node:
                for wall_at_node in walls_at_node:
                    if shell_name in all_t_intersects.keys():
                        if wall_at_node not in all_t_intersects[shell_name]:
                            all_t_intersects[shell_name].append(wall_at_node)
                    else:
                        all_t_intersects[shell_name]=[wall_at_node]
            elif shell_instance.detect_contact(node) and not shell_name in walls_at_node:
                for wall_at_node in walls_at_node:
                    if not wall_at_node in shell_bib.keys():
                        if shell_name in all_point_intersects.keys():
                            if wall_at_node not in all_point_intersects[shell_name]:
                                all_point_intersects[shell_name].append(wall_at_node)
                        else:
                            all_point_intersects[shell_name]=[wall_at_node]
    return all_nodes, all_t_intersects, all_point_intersects


def get_all_coincidents(
    current_model=[], 
    shell_bib={},
    all_bib ={},
):
    all_nodes,all_t_intersects,all_point_intersects=get_all_nodes(
        current_model=current_model, 
        shell_bib=shell_bib,
        all_bib =all_bib,
        )
    
    return {coord : walls for (coord, walls) in all_nodes.items() if len(walls)>1}


def get_all_coincident_pairs(
    current_model=[], 
    shell_bib={},
    all_bib ={},
):
    all_coincidents=get_all_coincidents(
        current_model=current_model, 
        shell_bib=shell_bib,
        all_bib =all_bib,
        )
    combination_bib={}
    for coord, walls in all_coincidents.items():
        combination_bib[coord]=[(a, b) for idx, a in enumerate(walls) for b in walls[idx + 1:]]
    return combination_bib


def get_foundation(
    current_model=[], 
    instance_bib=[]
    ):
    foundation_points=[]
    for key, wall in instance_bib.items():
        for node in current_model.rootAssembly.instances[key].vertices:
            coord,=node.pointOn
            if coord[1]==wall.Y and wall.foundation:
                vertex=current_model.rootAssembly.instances[key].vertices.findAt(coordinates=(((coord)),)) 
                foundation_points.append(vertex)
    return foundation_points

def get_datumplanes(current_model=[]):
    datumplanes_x={}
    datumplanes_y={}
    datumplanes_z={}
    for datum in current_model.rootAssembly.datums.values():
            try:
                if datum.normal == (1,0,0):
                    datumplanes_x[datum.pointOn[0]]=datum
                elif datum.normal == (0,1,0):
                    datumplanes_y[datum.pointOn[1]]=datum
                elif datum.normal == (0,0,1):
                    datumplanes_z[datum.pointOn[2]]=datum
            except:
                pass
    return datumplanes_x,datumplanes_y,datumplanes_z

def set_mode_nodes(sensor_data,model,instance_bib):
    import UTIL_assembly
    import mesh
    floor_nodes,_,_ = UTIL_assembly.get_all_nodes(
        current_model=model,
        shell_bib=instance_bib,
        all_bib =instance_bib,
        )
    for sensorname, data in sensor_data.items():
        closest=(10**12,None)
        for node, floors_at_node in floor_nodes.items():
            r_squared=0
            for a,b in zip(node,data["position"]):
                r_squared+=(a-b)**2
            if r_squared<closest[0]:
                closest=(r_squared,floors_at_node[0])
        if closest[1]+MODIFIED_SUFFIX in model.parts.keys():
            instance=model.rootAssembly.instances[closest[1]]
            part=model.parts[closest[1]+MODIFIED_SUFFIX]
        else:
            instance=model.rootAssembly.instances[closest[1]]
            part=model.parts[closest[1]]
        outputNode=[instance.nodes.getClosest(data["position"])]
        outputNodeArray = mesh.MeshNodeArray(nodes=outputNode)
        model.rootAssembly.Set(name=sensorname, nodes=outputNodeArray)