# -*- coding: mbcs -*-
#region Import and directory
from abaqus import *
from abaqusConstants import *
from part import *
from material import *
from section import *
from assembly import *
from step import *
from interaction import *
from load import *
from mesh import *
from optimization import *
from job import *
from sketch import *
from visualization import *
from connectorBehavior import *
from regionToolset import *
from odbAccess import *
#endregion


#TODO make the FREE mesh type specify medial axis, either by default or make it an option

def interpret_mesh_control(mesh_control_string):
    MESH_CONTROL_LOOKUP_MAP={
        "structured"        : STRUCTURED,
        "free"              : FREE,
        "quad"              : QUAD,
        "quad-dominated"    : QUAD_DOMINATED,     
        "tri"               : TRI,             
    } 
    assert(mesh_control_string.lower() in MESH_CONTROL_LOOKUP_MAP.keys(),"Could not find mesh control" + mesh_control_string +" in mesh_control_lookup_map")
    return MESH_CONTROL_LOOKUP_MAP[mesh_control_string.lower()]

def generate_mesh(current_model):
    current_model.rootAssembly.generateMesh(
        regions                 = current_model.rootAssembly.instances.values(),
        seedConstraintOverride  = False,
        meshTechniqueOverride   = False,
        )

def seed_instance_bib(
    current_model           = [],
    instance_bib            = [],             
):
    for instance_name, instance_object in instance_bib.items():
        assembly_instance   = current_model.rootAssembly.instances[instance_name]
        all_faces           = [face for face in assembly_instance.faces]
        all_edges           = [edge for edge in assembly_instance.edges]
        
        bounding_edges      = [edge.edge_object for edge in instance_object.get_instance_bounding_edges()]
        internal_edges      = [edge for edge in all_edges if edge not in bounding_edges]
        
        set_mesh_control(
            current_model,
            instance_object.
            mesh_controls,
            all_faces)
        
        seed_instance(
            current_model,
            instance_object.mesh_controls,
            assembly_instance,
            bounding_edges,
            internal_edges
            )

def set_mesh_control(current_model,mesh_controls,face_region):
    try:
        current_model.rootAssembly.setMeshControls(
            regions=face_region,
            technique=interpret_mesh_control(mesh_controls["Face structure"]),
            elemShape=interpret_mesh_control(mesh_controls["Element type"])
            )
    except:
        pass

def seed_instance(current_model,mesh_controls,assembly_instance,bounding_edges,internal_edges):
    current_model.rootAssembly.seedPartInstance(
            minSizeValue=mesh_controls["Global mesh size min"], 
            regions=(assembly_instance, ),
            size=mesh_controls["Global mesh size avg"])
    if mesh_controls["Bounding edge mesh size"]:
        current_model.rootAssembly.seedEdgeBySize(
            constraint  = FREE, 
            edges       = bounding_edges,
            size        = mesh_controls["Bounding edge mesh size"])
    if mesh_controls["Internal edge mesh size"]:
        current_model.rootAssembly.seedEdgeBySize(
            constraint  = FREE, 
            edges       = internal_edges,
            size        = mesh_controls["Internal edge mesh size"])

def flush_mesh(current_model):
    for instance in current_model.rootAssembly.instances.values():
        current_model.rootAssembly.deleteSeeds((instance.edges,) )
        current_model.rootAssembly.deleteMesh((instance,) )