# -*- coding: mbcs -*-
from CLASS_classes import SectionData
from abaqus import *
from abaqusConstants import *
from part import *
from material import *
from section import *
from assembly import *
from step import *
from interaction import *
from load import *
from mesh import *
from optimization import *
from job import *
from sketch import *
from visualization import *
from connectorBehavior import *
from regionToolset import *
from odbAccess import *
from sys import *
import openpyxl
import unicodedata

from CONF_config import *
from UTIL_tools                 import *
from UTIL_sets                  import *
import math



def define_section(
    current_model       = [],
    section_data        = {},
    ):
    
    if section_data["Type"] == 'CLT':
        sectiondata = define_CLT(
            current_model       = current_model,
            section_data         = section_data
            )
    elif section_data["Type"] == 'Concrete':
        sectiondata = define_concrete(
            current_model       = current_model,
            section_data        = section_data
            )
    elif section_data["Type"] == 'HUB':
        sectiondata = define_HUB(
            current_model       = current_model,
            section_data        = section_data
            )
    return sectiondata

def define_CLT(
    current_model       = [],
    section_data        = {}
    ):
    x_thickness=0
    y_thickness=0
    layup=[]
    for i in range(1,(len(section_data.keys())-2)/3+1):
        if section_data["Thickness "+str(i)]:
            x_thickness+=math.cos(math.radians(section_data["Orientation "+str(i)]))*section_data["Thickness "+str(i)]
            y_thickness+=math.sin(math.radians(section_data["Orientation "+str(i)]))*section_data["Thickness "+str(i)]
            layup.append(
                SectionLayer(
                        thickness       = section_data["Thickness "+str(i)], 
                        orientAngle     = section_data["Orientation "+str(i)], 
                        material        = section_data["Material "+str(i)],
                        plyName         ='Ply ' + str(i)
                        )
                )
    current_model.CompositeShellSection(
        idealization=NO_IDEALIZATION, 
        integrationRule=SIMPSON, 
        layup=tuple(layup), 
        name=section_data["Name"],
        poissonDefinition=DEFAULT, 
        preIntegrate=OFF, 
        symmetric=False, 
        temperature=GRADIENT, 
        thicknessModulus=None, 
        thicknessType=UNIFORM, 
        useDensity=OFF
        )
    return SectionData(section_data["Name"],section_data["Material 1"],(x_thickness+y_thickness),x_thickness,y_thickness)

def define_concrete(
    current_model       = [],
    section_data        = []
    ):
    current_model.HomogeneousShellSection(
        idealization=NO_IDEALIZATION, 
        integrationRule=SIMPSON, 
        material=section_data["Material 1"], 
        name=section_data["Name"], 
        nodalThicknessField='', 
        numIntPts=5, 
        poissonDefinition=DEFAULT, 
        preIntegrate=OFF, 
        temperature=GRADIENT, 
        thickness=section_data["Thickness 1"], 
        thicknessField='', 
        thicknessModulus=None, 
        thicknessType=UNIFORM, 
        useDensity=OFF
        )
    return SectionData(section_data["Name"],section_data["Material 1"],section_data["Thickness 1"],section_data["Thickness 1"],section_data["Thickness 1"])
def define_HUB(
    current_model       = [],
    section_data        = []
    ):
    current_model.BoxProfile(
        a=section_data["b"], 
        b=section_data["c"], 
        name="Profile "+section_data["Name"], 
        t1=section_data["d"], 
        uniformThickness=ON
        )
    current_model.BeamSection(
        consistentMassMatrix=False, 
        integration=DURING_ANALYSIS, 
        material=section_data["a"], 
        name=section_data["Name"], 
        poissonRatio=0.0, 
        profile="Profile "+section_data["Name"], 
        temperatureVar=LINEAR
        )
    return SectionData(section_data["Name"],section_data["a"],None,None,None)


def section_assignment_in_parts(
    current_model   = [],
    part            = '',           #The part in which you want the section assigned. 
    #set_to_assign   = '',           #The name of a pre-existing set.  
    section_Name    = ''            #The name of the section you want to assign
):
    current_model.parts[part].SectionAssignment(
        region = current_model.parts[part],   #.sets[set_to_assign],
        sectionName = section_Name)



