classdef FDD
   methods (Static)
        function [files,data_dir,result_dir]=find_files(building,freq)
            if building == "SML3"
                if freq == 40
                    data_dir = '..\data_Tromso2021_processed\SML3\40hz\';
                    result_dir = '..\data_Tromso2021_calulations\SML3\40hz\';
                else
                    data_dir = '..\data_Tromso2021_processed\SML3\20hz\';
                    result_dir =  '..\data_Tromso2021_calulations\SML3\20hz\';
                end
            elseif building =="SML5"
                if freq == 40
                    data_dir = '..\data_Tromso2021_processed\SML5\40hz\';
                    result_dir = '..\data_Tromso2021_calulations\SML5\40hz\';
                else
                    data_dir = '..\data_Tromso2021_processed\SML5\20hz\';
                    result_dir =  '..\data_Tromso2021_calulations\SML5\20hz\';
                end
            end
            files = dir(strcat(data_dir,'*.mat'))';
        end
        function [data_timestamp,data,winddata_timestamp,winddir,windmag]=load_data(data_dir,file)
            file_path=strcat(data_dir,file.name);
            load(file_path);
            data_timestamp=acceleration_datatimestamp;
            winddata_timestamp=metadata_sensor_mod7_ch00_nametimestamp;
            winddir=metadata_sensor_mod7_ch00_namedirection;
            windmag=metadata_sensor_mod7_ch00_namemagnitude;
            % Select channels
            data=[
                acceleration_dataA1_X
                acceleration_dataA2_X
                acceleration_dataA3_X
                acceleration_dataA4_X
                 acceleration_dataA1_Y
                 acceleration_dataA2_Y
                 acceleration_dataA3_Y
                 acceleration_dataA4_Y
                acceleration_dataA1_Z
                acceleration_dataA2_Z
                acceleration_dataA3_Z
                acceleration_dataA4_Z
                ]';
        end
        %PSD 
        function [PSD,frequency_range] = compute_PSD(data,psd_parameters)
            %Randomize overlap ratio
            noverlap = round(psd_parameters.window*psd_parameters.overlap_percentage/100,0);
            
            %Compute PSD
            PSD= zeros(size(data,2),size(data,2),psd_parameters.nfft);   
            for I=1:size(data,2)
                for J=1:size(data,2)
                    [PSD(I,J,:),F(I,J,:)]=cpsd(data(:,I),data(:,J),psd_parameters.window,noverlap,psd_parameters.n_freq,psd_parameters.fs);
                end
            end
            frequency_range(:,1)=F(1,1,:);
        end
        %SVD
        function [U,S] = compute_SVD(PSD)
            U = zeros(size(PSD,1),size(PSD,1),size(PSD,3));
            S = zeros(size(PSD,1),size(PSD,1),size(PSD,3));
            for k=1:size(PSD,3)
                [u,s,~]=svd(PSD(:,:,k));
                [U(:,:,k)] = U(:,:,k)+u; %Mode shapes
                [S(:,:,k)] = S(:,:,k)+s; %Eigen values
            end
        end
        %Mode identification
        function [found_peaks_values,found_peaks_indicies, found_frequencies] = identify_modes(spectrum,approx_freqs,search_window_width,df)
            found_peaks_values      = [];
            found_peaks_indicies    = [];
            found_frequencies       = [];
            for n = 1:length(approx_freqs)
                %Convert to indices
                search_idx=round(approx_freqs(n)/df,0);
                search_window=round(search_window_width/df,0);
                %Create a small intervall around approx mode freq
                snippet=spectrum((search_idx-search_window):(search_idx+search_window));
                %Find index of toppoint
                [value,index]=max(snippet);
                %Store values
                found_peaks_values(n)   = value;
                found_peaks_indicies(n) = (search_idx-search_window)+index-1;
                found_frequencies(n)    = found_peaks_indicies(n)*df;
            end
        end
        %Damping estimation
        function R_norm=compute_corr_norm(sigma,PSD_PARAMETERS)
            nifft           = size(sigma,2)*PSD_PARAMETERS.ifft_padding_factor;          
            R               = zeros(size(sigma,1),nifft);
            R_norm          = zeros(size(sigma,1),nifft);
            for j = 1:size(sigma,1)
               R(j,:)   = (ifft(sigma(j,:),nifft,'symmetric'));   
               R_norm(j,:)  = (R(j,:)-mean(R(j,:)))/R(j,1);
            end
        end
        function [sigma,mode_estimate] = define_peak_area(mac_criterion,peak,U,S,num_peaks,num_dofs,n_freq)
            mode_estimate = zeros(num_dofs,num_peaks);
            for j = 1:num_peaks
                %Identify target modes
                mode_estimate(:,j) = U(:,1,peak(j));
            end
            sigma = zeros(num_peaks,n_freq);
            for i = 1:num_peaks
                j = peak(i);
                for n = 1:num_dofs
                    %Walk right until MAC is too low
                    for k = j:1:n_freq
                        if FDD.compute_MAC(U(:,n,k), mode_estimate(:,i))>= mac_criterion
                            sigma(i,k) = sigma(i,k) + S(n,n,k);
                        else
                            break
                        end
                    end
                    %Walk left until MAC is too low
                    for k = j-1:-1:1
                        if FDD.compute_MAC(U(:,n,k), mode_estimate(:,i))>= mac_criterion
                            sigma(i,k) = sigma(i,k) + S(n,n,k);
                        else
                            break
                        end
                    end
                end
            end
        end           
        function MAC_value = compute_MAC(mode1, mode2)
                numerator   = abs(mode1'*mode2)^2; 
                divisor     = (mode1'*mode1)*(mode2'*mode2);
                MAC_value	= numerator/divisor;
        end
        function [locsdof,peaksdof]=locate_peaks(fig,correlation_matrix,rejection_threshold)
            peaksdof={};
            locsdof ={};
           
            figure(fig)
            t = tiledlayout(size(correlation_matrix,1),1);
            for i = 1:size(correlation_matrix,1)
                [peaks,locs]=findpeaks(abs(correlation_matrix(i,:)));
                start_index=1;
                for j = 1:length(peaks)
                    peaksdof{i}  = peaks(start_index:j);
                    locsdof{i}   = locs(start_index:j);
                    if peaks(j)>=rejection_threshold(1)
                        start_index=j;
                    end
                    if peaks(j)<rejection_threshold(2)
                        break
                    end
                end
                nexttile
                plot(abs(correlation_matrix(i,:)))
                hold on
                plot(locsdof{i},peaksdof{i},'or')
                hold off
                ylim([0 1])
                xlim([1 700])
            end
        end
        function delta = compute_delta(locsdof,peaksdof,autocorr)
            delta = zeros(1,length(locsdof));
            for k = 1:length(locsdof)
                delta(k) = (2/k) * log(autocorr/abs(peaksdof(k)));
            end
        end
        function [locsdof,peaksdof,delta]=compute_delta_struct(fig,corr_norm,REJECTION_THRESHOLD)
            [locsdof,peaksdof]=FDD.locate_peaks(fig,corr_norm,REJECTION_THRESHOLD);
            for n=1:size(corr_norm,1)
                delta{n}=FDD.compute_delta(locsdof{n},peaksdof{n},corr_norm(n,n));
            end
        end
        function [zeta]=compute_zeta(fig,locsdof,peaksdof,delta)
            zeta=[];
            figure(fig)
            t = tiledlayout(size(delta,2),1);

            for i=1:size(delta,2)
                Y = delta{i}.*locsdof{i} + 2*log(abs(peaksdof{i}));
                P = polyfit(locsdof{i},Y,1);
                zeta(i) = abs(P(1)/sqrt(P(1)^2+(2*pi)^2));
                
                
                nexttile
                plot(locsdof{i},Y,'-k','LineWidth',1.5)
                hold on
                plot(locsdof{i},P(1).*locsdof{i}+P(2),':r','LineWidth',1.5)
                
                % Add title
                
                tittel_temp = "$\hat{\zeta}$ = "+num2str(zeta(i)*100,"%.2f")+" \%";
                title(tittel_temp,'Interpreter','latex')
                
            end
        end
        % Results and plots
        function present_modes(picked_frequencies, mode_shapes)
            disp('Modes')
            for I=1:size(picked_frequencies,2)
                fprintf('%.3f Hz\n',picked_frequencies(I))
                disp(mode_shapes(:,I))
            end
        end
   end
end
