close all
output_dir= fullfile('FDD_output\');
filename = 'FDD_results.csv';

%% Processing parameters 
% DATA PARAMETERS
DATA_PARAMETERS.building            = "SML3";
DATA_PARAMETERS.IDX_START           = 1; %FILE TO START AT
DATA_PARAMETERS.segment_length      = 60; %minutes
DATA_PARAMETERS.NUM_INCLUDE         = 302*(360/DATA_PARAMETERS.segment_length); %NUMBER OF SEGMENTS
% PSD PARAMETERS
PSD_PARAMETERS.fs                   = 20;
PSD_PARAMETERS.windowlength         = DATA_PARAMETERS.segment_length/10; %minutes
PSD_PARAMETERS.window               = PSD_PARAMETERS.fs*60*PSD_PARAMETERS.windowlength;
PSD_PARAMETERS.overlap_percentage   = 50;
PSD_PARAMETERS.nfft                 = PSD_PARAMETERS.window;
PSD_PARAMETERS.ifft_padding_factor	= 160/PSD_PARAMETERS.fs;
PSD_PARAMETERS.df                   = PSD_PARAMETERS.fs/(2*PSD_PARAMETERS.nfft);
PSD_PARAMETERS.n_freq               = 2*PSD_PARAMETERS.nfft-1;

% MODE PARAMETERS
MODE_PARAMETERS.mac_criterion       = 0.80;

if DATA_PARAMETERS.building == "SML3"
    PSD_PARAMATERS.sample_noise_profile = [3 4];
    MODE_PARAMETERS.mode_search         = [
                                            1.300
                                            1.635
                                            1.995
                                            ];
elseif DATA_PARAMETERS.building == "SML5"
    PSD_PARAMATERS.sample_noise_profile = [4 5];
    MODE_PARAMETERS.mode_search         = [
                                            2.160
                                            2.450
                                            2.930
                                            ];
end
PSD_PARAMETERS.sample_noise_range   = round(PSD_PARAMATERS.sample_noise_profile(1)/PSD_PARAMETERS.df):PSD_PARAMATERS.sample_noise_profile(2)/PSD_PARAMETERS.df;

MODE_PARAMETERS.mode_search_window  = 0.1;%Hz
% REGRESSION PARAMETERS
REJECTION_THRESHOLD                 = [0.95 0.4];


%% Get path

[files,data_dir,result_dir]=FDD.find_files(DATA_PARAMETERS.building,PSD_PARAMETERS.fs);
num_files=length(files);

%Initializing a bunch of lists to store data in
names                   = [];
noise_floors            = [];
damping_my              = [];
mode_peak_excitation    = [];
mode_mean_excitation    = [];
mode_rms_excitation     = [];
mode_frequencies        = [];
mode_shapes             = [];
wind_mag_mu             = [];
wind_mag_RMS            = [];
wind_mag_std            = [];
datatimestamps          = [];
windtimestamps          = [];    
output_table            = [];

IDX_STOP = min([(DATA_PARAMETERS.IDX_START +DATA_PARAMETERS.NUM_INCLUDE),num_files]);
%% Loop over files
filecounter=DATA_PARAMETERS.IDX_START-1;
segment_counter=0;
figures               = setup_figures();
while filecounter<num_files && segment_counter<DATA_PARAMETERS.NUM_INCLUDE
    filecounter=filecounter+1;
    % Load data
    [data_timestamp,data,winddata_timestamp,winddir,windmag]=FDD.load_data(data_dir,files(filecounter));
    num_segments = round(length(data_timestamp)/(DATA_PARAMETERS.segment_length*60*PSD_PARAMETERS.fs),0);
    data_segment_length=floor(length(data_timestamp)/(num_segments));
    wind_segment_length=floor(length(winddata_timestamp)/(num_segments));
    total_segments = floor(min(num_files*num_segments,DATA_PARAMETERS.NUM_INCLUDE));
    small_counter=0;
    while segment_counter<DATA_PARAMETERS.NUM_INCLUDE && small_counter<num_segments
        %Ptogress tracking
        small_counter=small_counter+1;
        segment_counter=segment_counter+1;
        
        %Cut off indices for the different data sources
        data_start=(small_counter-1)*data_segment_length+1;
        data_stop=data_start+data_segment_length-1;
        wind_start=(small_counter-1)*wind_segment_length+1;
        wind_stop=wind_start+wind_segment_length-1;
        
        % Compute the cross-spectral density matrix
        [PSD,frequency_range]  = FDD.compute_PSD(data(data_start:data_stop,:),PSD_PARAMETERS);
        
        % Singular value decompoition of the PSD matrix
        [U,S] = FDD.compute_SVD(PSD);
        
        % Plotting the 1st order periodogram
        figure(figures(1))
        clf
        s1=[];
        s1(1,:) = reshape(S(1,1,:),1,[]);
        semilogy(frequency_range,s1)
        
        %Computing the level of noise in the data
        noise_floor =mean(s1(PSD_PARAMETERS.sample_noise_range));
        noise_floors(segment_counter)=noise_floor;
        
        %Finding the peaks around hardcoded areas to search, and ploting
        %them onto the psd plot
        hold on
        [found_peaks_values,found_peaks_indicies, found_frequencies] = FDD.identify_modes(s1,MODE_PARAMETERS.mode_search,MODE_PARAMETERS.mode_search_window,PSD_PARAMETERS.df);
        plot(found_frequencies,found_peaks_values,'or')
        for i = 1:length(found_frequencies)
            text(found_frequencies(i)-0.3,max(found_peaks_values)*1.7^(i),num2str(found_frequencies(i),"%.3f"),'Color','red','FontSize',10)
        end
        ylim([mean(s1)/10, 10*max(s1)])
        pause(0.5)
        
        %Performing the math to compute damping values
        [zeta_results,mode_estimate,sigma]=estimate_damping(figures,found_peaks_indicies,U,S,length(found_peaks_indicies),size(data(data_start:data_stop,:),2),MODE_PARAMETERS,PSD_PARAMETERS,REJECTION_THRESHOLD,frequency_range);
        
        %Plotting it to console
        FDD.present_modes(found_frequencies, mode_estimate)
        
        %Storing the current results in a master list for later use
        mode_shapes(segment_counter,:,:)=mode_estimate;
        mode_frequencies(segment_counter,:)=found_frequencies;
        names{segment_counter}     = string(files(filecounter).name);
        damping_my(segment_counter,:) = [zeta_results(1) zeta_results(2) zeta_results(3)];
        mode_peak_excitation(segment_counter,:) = found_peaks_values/noise_floor; 
        mode_mean_excitation(segment_counter,:) = mean(sigma')/noise_floor; 
        mode_rms_excitation(segment_counter,:) = rms(sigma')/noise_floor; 
        wind_mag_RMS(segment_counter) = sqrt(mean((windmag(wind_start:wind_stop)).^2));
        wind_mag_mu(segment_counter) = mean(windmag(wind_start:wind_stop));
        wind_mag_std(segment_counter) = mean(windmag(wind_start:wind_stop));
        datatimestamps(segment_counter)= data_timestamp(data_start);
        windtimestamps(segment_counter)= winddata_timestamp(wind_start); 
        
        %Progress tracking again
        progress=round((segment_counter/total_segments)*100,1);
        fprintf("\n%.0f %%\n",progress)
    end
end

%Formating the data and preparing to save it as .csv file
wind_table = table(names',windtimestamps',wind_mag_mu',wind_mag_RMS',wind_mag_std',noise_floors','VariableNames',{'inpfilename','wind_time_stamp','wind_mag_mu','wind_mag_RMS','wind_mag_std','psd_noise_floor'});
damping_table=table(datatimestamps','VariableNames',{'data_time_stamp'});
for i = 1:size(mode_estimate,2)
    variablenames=[[
        "mode_peak_excitation"
        "mode_mean_excitation"
        "mode_rms_excitation"
        "mode_frequency"
        "damping_ratio"
        ]' strcat("node_",sprintfc('%d',1:size(data,2)))];  
    damping_table = [damping_table cell2table([num2cell([mode_peak_excitation(:,i),mode_mean_excitation(:,i),mode_rms_excitation(:,i),mode_frequencies(:,i),damping_my(:,i), real(mode_shapes(:,:,i))])],'VariableNames',strcat("mode_",num2str(i),"_",variablenames))];     
end
file_path=strcat(output_dir,DATA_PARAMETERS.building,"\",DATA_PARAMETERS.building,"-",num2str(PSD_PARAMETERS.fs),"Hz","-",num2str(DATA_PARAMETERS.segment_length),"min",'.csv');
writetable([wind_table damping_table],file_path);

%% FUNCTIONS
function figure_list            = setup_figures()
    % Just a function that builds a bunch of figures for creating a
    % dashboard-like window. Nice to have since it gives visualisation of
    % the quality of data and the process in general.
    PSD_fig         = figure(1);
    Peak_PSD_fig    = figure(2);
    auto_corr_fig   = figure(3);
    zeta_fig        = figure(4);
    figure_list=[
        PSD_fig
        Peak_PSD_fig
        auto_corr_fig
        zeta_fig
        ];
    set(figure_list, 'WindowStyle', 'Docked');
end

function [zeta_results,mode_estimate,sigma]=estimate_damping(figures,peaks,U,S,num_peaks,num_channels,mode_parameters,psd_parameters,rejection_threshold,frequency_range)
    %This is the master algorithm for the efdd technique
    
    %First we define an area around each mode peak, in this case using the
    %MAC-criterion
    [sigma, mode_estimate]      = FDD.define_peak_area(mode_parameters.mac_criterion,peaks,U,S,num_peaks,num_channels,psd_parameters.nfft);
    
    %Ploting to the dashboard
    figure(figures(2))
    clf
    for i =1:size(sigma,1)
        semilogy(frequency_range,sigma(i,:));
        hold on
    end
    
    %Now we compute the ifft of the small ranges around the peaks
    corr_norm                   = FDD.compute_corr_norm(sigma,psd_parameters);
    
    %This function identifies the individual peaks of the auto-correlations
    %and computes the logarithmic decrement of them
    [locsdof,peaksdof,delta]    = FDD.compute_delta_struct(figures(3),corr_norm,rejection_threshold);
    
    %This function computes the damping ratio by use of linear regression
    %of the logarithmic decrement to achieve a more stable result.
    zeta_results                = FDD.compute_zeta(figures(4),locsdof,peaksdof,delta);
end
