close all
clear all
clf

output_dir= fullfile('FDD_output\');
filename = 'FDD_results.csv';


%% Processing parameters 
% DATA PARAMETERS
DATA_PARAMETERS.building            = "SML3";
DATA_PARAMETERS.IDX_START           = 47; %FILE TO START AT
DATA_PARAMETERS.segment_length      = 30; %minutes
DATA_PARAMETERS.NUM_INCLUDE         = 4*(360/DATA_PARAMETERS.segment_length); %NUMBER OF SEGMENTS
% PSD PARAMETERS
PSD_PARAMETERS.fs                   = 20;
PSD_PARAMETERS.windowlength         = DATA_PARAMETERS.segment_length/10; %minutes
PSD_PARAMETERS.window               = PSD_PARAMETERS.fs*60*PSD_PARAMETERS.windowlength;
PSD_PARAMETERS.overlap_percentage   = 50;
PSD_PARAMETERS.nfft                 = PSD_PARAMETERS.window;
PSD_PARAMETERS.df                   = PSD_PARAMETERS.fs/(2*PSD_PARAMETERS.nfft);
PSD_PARAMETERS.n_freq               = 2*PSD_PARAMETERS.nfft-1;

if DATA_PARAMETERS.building == "SML3"
    MODE_PARAMETERS.mode_search         = [
                                            1.300
                                            1.625
                                            2.000
                                            4.850
                                            5.600
                                            6.050
                                            6.400
                                            ];
elseif DATA_PARAMETERS.building == "SML5"
    MODE_PARAMETERS.mode_search         = [
                                            2.150
                                            2.450
                                            2.925
                                            %6.885
                                            ];
end
MODE_PARAMETERS.mode_search_window  = 0.1;%Hz


%% Get path
[files,data_dir,result_dir]=FDD.find_files(DATA_PARAMETERS.building,PSD_PARAMETERS.fs);
num_files=length(files);

IDX_STOP = min([(DATA_PARAMETERS.IDX_START +DATA_PARAMETERS.NUM_INCLUDE),num_files]);
%% Loop over files
filecounter=DATA_PARAMETERS.IDX_START-1;
segment_counter=0;

while filecounter<num_files && segment_counter<DATA_PARAMETERS.NUM_INCLUDE
    filecounter=filecounter+1;
    % Load data
    [data_timestamp,data,winddata_timestamp,winddir,windmag]=FDD.load_data(data_dir,files(filecounter));
    num_segments = round(length(data_timestamp)/(DATA_PARAMETERS.segment_length*60*PSD_PARAMETERS.fs),0);
    data_segment_length=floor(length(data_timestamp)/(num_segments));
    total_segments = floor(min(num_files*num_segments,DATA_PARAMETERS.NUM_INCLUDE));
    small_counter=0;
    
    psdsum = zeros(1,PSD_PARAMETERS.nfft);
    while segment_counter<DATA_PARAMETERS.NUM_INCLUDE && small_counter<num_segments
        small_counter=small_counter+1;
        segment_counter=segment_counter+1;

        data_start=(small_counter-1)*data_segment_length+1;
        data_stop=data_start+data_segment_length-1;
       
        % Cross-spectral density
        [PSD,frequency_range]  = FDD.compute_PSD(data(data_start:data_stop,:),PSD_PARAMETERS);
        [U,S] = FDD.compute_SVD(PSD);
        s1=[];
        s1(1,:) = reshape(S(1,1,:),1,[]);     
        psdsum = psdsum + s1;
        psd = psdsum/segment_counter;
        
        %Plot and find peaks
        clf
        semilogy(frequency_range,psd)
        hold on
        [found_peaks_values,found_peaks_indicies, found_frequencies] = FDD.identify_modes(psd,MODE_PARAMETERS.mode_search,MODE_PARAMETERS.mode_search_window,PSD_PARAMETERS.df);
        plot(found_frequencies,found_peaks_values,'or')
        for i = 1:length(found_frequencies)
            textx=found_frequencies(i)-0.25;
            texty=found_peaks_values(i)*1.7^(i);
            plot([found_frequencies(i),textx+0.25],[found_peaks_values(i),texty*0.7],'Color','black')
            text(textx,texty,num2str(found_frequencies(i),"%.3f"),'Color','red','FontSize',20)
        end   
        xlim([0,10])
        ylim([10e-10, max(found_peaks_values)*50]);
        title(strcat("Power spectral density ",DATA_PARAMETERS.building),'FontSize',30)
        xlabel("f [Hz]",'FontSize',20)
        set(gca,'ytick',[])
        ax = gca;
        ax.FontSize = 16;
        pause(0.05) 
        progress=round((segment_counter/total_segments)*100,1);
        fprintf("\n%.0f %%\n",progress)
    end
end